@@ -2,7 +2,7 @@
 # Copyright (c) 2022 MBition GmbH
 
 from pathlib import Path
-from typing import List, Optional
+from typing import List, Set, Optional
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element
 from itertools import chain
@@ -26,8 +26,7 @@ class Database:
 
     def __init__(self,
                  pdx_zip: Optional[ZipFile] = None,
-                 odx_d_file_name: Optional[str] = None,
-                 enable_candela_workarounds: bool = True):
+                 odx_d_file_name: Optional[str] = None):
 
         if pdx_zip is None and odx_d_file_name is None:
             # create an empty database object
@@ -62,8 +61,7 @@ class Database:
             dlc = root.find("DIAG-LAYER-CONTAINER")
             if dlc is not None:
                 dlcs.append(read_diag_layer_container_from_odx(
-                    dlc,
-                    enable_candela_workarounds=enable_candela_workarounds
+                    dlc
                 ))
             # In ODX 2.0 there was only COMPARAM-SPEC
             # In ODX 2.2 content of COMPARAM-SPEC was renamed to COMPARAM-SUBSET
@@ -97,24 +95,24 @@ class Database:
         self._odxlinks = OdxLinkDatabase()
 
         for subset in self.comparam_subsets:
-            self.odxlinks.update(subset._build_odxlinks())
+            self._odxlinks.update(subset._build_odxlinks())
 
         for dlc in self.diag_layer_containers:
-            self.odxlinks.update(dlc._build_odxlinks())
+            self._odxlinks.update(dlc._build_odxlinks())
 
         for dl in self.diag_layers:
-            self.odxlinks.update(dl._build_odxlinks())
+            self._odxlinks.update(dl._build_odxlinks())
 
         # Resolve references
         for subset in self.comparam_subsets:
-            subset._resolve_references(self.odxlinks)
+            subset._resolve_references(self._odxlinks)
         for dlc in self.diag_layer_containers:
-            dlc._resolve_references(self.odxlinks)
+            dlc._resolve_references(self._odxlinks)
 
         for dl_type_name in ["ECU-SHARED-DATA", "PROTOCOL", "FUNCTIONAL-GROUP", "BASE-VARIANT", "ECU-VARIANT"]:
             for dl in self.diag_layers:
                 if dl.variant_type == dl_type_name:
-                    dl._resolve_references(self.odxlinks)
+                    dl._resolve_references(self._odxlinks)
 
     @property
     def odxlinks(self) -> OdxLinkDatabase:
@@ -141,4 +139,16 @@ class Database:
 
     @property
     def comparam_subsets(self):
-        return self._comparam_subsets
\ No newline at end of file
+        return self._comparam_subsets
+
+    @property
+    def protocol_names(self) -> Set[str]:
+        """
+        The names of all protocols defined for this database
+        """
+        result = set()
+        for dlc in self.diag_layer_containers:
+            for prot in dlc.protocols:
+                result.add(prot.short_name)
+
+        return result
