@@ -3,7 +3,7 @@
 
 import abc
 from typing import cast, List, Dict, Optional, Any, Union
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 
 from .utils import read_description_from_odx
 from .physicaltype import PhysicalType, read_physical_type_from_odx
@@ -14,10 +14,11 @@ from .diagcodedtypes import DiagCodedType, StandardLengthType, read_diag_coded_t
 from .decodestate import DecodeState
 from .encodestate import EncodeState
 from .exceptions import DecodeError, EncodeError
+from .specialdata import SpecialDataGroup, read_sdgs_from_odx
 from .odxlink import OdxLinkRef, OdxLinkId, OdxDocFragment, OdxLinkDatabase
 
 class DopBase(abc.ABC):
-    """ Base class for all DOPs.
+    """Base class for all DOPs.
 
     Any class that a parameter can reference via a DOP-REF should inherit from this class.
     """
@@ -27,12 +28,26 @@ class DopBase(abc.ABC):
                  short_name,
                  long_name=None,
                  description=None,
-                 is_visible=True):
+                 is_visible=True,
+                 sdgs = []):
         self.odx_id = odx_id
         self.short_name = short_name
         self.long_name = long_name
         self.description = description
         self.is_visible = is_visible
+        self.sdgs = sdgs
+
+    def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
+        result = {}
+
+        for sdg in self.sdgs:
+            result.update(sdg._build_odxlinks())
+
+        return result
+
+    def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
+        for sdg in self.sdgs:
+            sdg._resolve_references(odxlinks)
 
     @abc.abstractmethod
     def convert_physical_to_bytes(self, physical_value, encode_state: EncodeState, bit_position: int) -> bytes:
@@ -56,12 +71,14 @@ class DataObjectProperty(DopBase):
                  compu_method: CompuMethod,
                  unit_ref: Optional[OdxLinkRef] = None,
                  long_name: Optional[str] = None,
-                 description: Optional[str] = None
+                 description: Optional[str] = None,
+                 sdgs: List[SpecialDataGroup] = [],
                  ):
         super().__init__(odx_id=odx_id,
                          short_name=short_name,
                          long_name=long_name,
-                         description=description)
+                         description=description,
+                         sdgs=sdgs)
         self.diag_coded_type = diag_coded_type
         self.physical_type = physical_type
         self.compu_method = compu_method
@@ -120,6 +137,8 @@ class DataObjectProperty(DopBase):
 
     def _resolve_references(self, odxlinks: OdxLinkDatabase):
         """Resolves the reference to the unit"""
+        super()._resolve_references(odxlinks)
+
         if self.unit_ref:
             self._unit = odxlinks.resolve(self.unit_ref)
 
@@ -155,7 +174,19 @@ class DiagnosticTroubleCode:
     display_trouble_code: Optional[str] = None
     level: Union[bytes, bytearray, None] = None
     is_temporary: bool = False
+    sdgs: List[SpecialDataGroup] = field(default_factory=list)
 
+    def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
+        result = {}
+
+        for sdg in self.sdgs:
+            result.update(sdg._build_odxlinks())
+
+        return result
+
+    def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
+        for sdg in self.sdgs:
+            sdg._resolve_references(odxlinks)
 
 class DtcRef:
     """A proxy for DiagnosticTroubleCode.
@@ -196,10 +227,11 @@ class DtcRef:
         return self.dtc.is_temporary
 
     def _resolve_references(self, odxlinks: OdxLinkDatabase):
-        self.dtc: Optional[DiagnosticTroubleCode] = odxlinks.resolve(self.dtc_ref) # type: ignore
-
-        assert isinstance(self.dtc, DiagnosticTroubleCode),\
-            f"DTC-REF {self.dtc_ref} does not reference a DTC but a {type(self.dtc)}."
+        dtc = odxlinks.resolve(self.dtc_ref)
+        assert isinstance(dtc, DiagnosticTroubleCode),\
+            f"DTC-REF {self.dtc_ref} references an object of type {type(dtc)} " \
+            f"instead of a DiagnosticTroubleCode."
+        self.dtc = dtc
 
 
 class DtcDop(DataObjectProperty):
@@ -215,7 +247,8 @@ class DtcDop(DataObjectProperty):
                  is_visible: bool = False,
                  linked_dtc_dops: bool = False,
                  long_name: Optional[str] = None,
-                 description: Optional[str] = None
+                 description: Optional[str] = None,
+                 sdgs: List[SpecialDataGroup] = [],
                  ):
         super().__init__(odx_id=odx_id,
                          short_name=short_name,
@@ -223,7 +256,8 @@ class DtcDop(DataObjectProperty):
                          description=description,
                          diag_coded_type=diag_coded_type,
                          physical_type=physical_type,
-                         compu_method=compu_method)
+                         compu_method=compu_method,
+                         sdgs=sdgs)
         self.dtcs = dtcs
         self.is_visible = is_visible
         self.linked_dtc_dops = linked_dtc_dops
@@ -268,19 +302,21 @@ class DtcDop(DataObjectProperty):
         return super().convert_physical_to_bytes(trouble_code, encode_state, bit_position)
 
     def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
-        odxlinks: Dict[OdxLinkId, Any] = {}
+        odxlinks = super()._build_odxlinks()
         odxlinks[self.odx_id] = self
         for dtc in self.dtcs:
             if isinstance(dtc, DiagnosticTroubleCode):
                 assert dtc.odx_id is not None
                 odxlinks[dtc.odx_id] = dtc
+                odxlinks.update(dtc._build_odxlinks())
+
         return odxlinks
 
     def _resolve_references(self, odxlinks: OdxLinkDatabase):
-        for dtc in self.dtcs:
-            if isinstance(dtc, DtcRef):
-                dtc._resolve_references(odxlinks)
+        super()._resolve_references(odxlinks)
 
+        for dtc in self.dtcs:
+            dtc._resolve_references(odxlinks)
 
 def read_dtc_from_odx(et_element, doc_frags: List[OdxDocFragment]):
     if et_element.find("DISPLAY-TROUBLE-CODE") is not None:
@@ -293,13 +329,16 @@ def read_dtc_from_odx(et_element, doc_frags: List[OdxDocFragment]):
     else:
         level = None
 
+    sdgs = read_sdgs_from_odx(et_element.find("SDGS"), doc_frags)
+
     return DiagnosticTroubleCode(odx_id=OdxLinkId.from_et(et_element, doc_frags),
                                  short_name=et_element.findtext("SHORT-NAME"),
                                  trouble_code=int(
                                      et_element.findtext("TROUBLE-CODE")),
                                  text=et_element.findtext("TEXT"),
                                  display_trouble_code=display_trouble_code,
-                                 level=level)
+                                 level=level,
+                                 sdgs=sdgs)
 
 
 def read_data_object_property_from_odx(et_element, doc_frags: List[OdxDocFragment]) \
@@ -310,7 +349,7 @@ def read_data_object_property_from_odx(et_element, doc_frags: List[OdxDocFragmen
     short_name = et_element.findtext("SHORT-NAME")
     long_name = et_element.findtext("LONG-NAME")
     description = read_description_from_odx(et_element.find("DESC"))
-    logger.debug('Parsing DOP ' + short_name)
+    sdgs = read_sdgs_from_odx(et_element.find("SDGS"), doc_frags)
 
     diag_coded_type = read_diag_coded_type_from_odx(
         et_element.find("DIAG-CODED-TYPE"), doc_frags)
@@ -329,7 +368,8 @@ def read_data_object_property_from_odx(et_element, doc_frags: List[OdxDocFragmen
                                  compu_method,
                                  unit_ref=unit_ref,
                                  long_name=long_name,
-                                 description=description)
+                                 description=description,
+                                 sdgs=sdgs)
     else:
         dtcs = [read_dtc_from_odx(el, doc_frags)
                 for el in et_element.iterfind("DTCS/DTC")]
@@ -345,5 +385,6 @@ def read_data_object_property_from_odx(et_element, doc_frags: List[OdxDocFragmen
                      dtcs,
                      is_visible=is_visible,
                      long_name=long_name,
-                     description=description)
+                     description=description,
+                     sdgs=sdgs)
     return dop
