@@ -5,6 +5,8 @@ from itertools import chain
 from typing import Optional, Any, Dict, Iterable, List, Union
 from copy import copy
 import warnings
+from xml.etree import ElementTree
+from deprecated import deprecated
 
 from .utils import short_name_as_id
 from .exceptions import DecodeError, OdxWarning
@@ -17,6 +19,7 @@ from .utils import read_description_from_odx
 from .nameditemlist import NamedItemList
 from .admindata import AdminData, read_admin_data_from_odx
 from .companydata import CompanyData, read_company_datas_from_odx
+from .comparam_subset import Comparam, ComplexComparam
 from .communicationparameter import CommunicationParameterRef, read_communication_param_ref_from_odx
 from .diagdatadictionaryspec import DiagDataDictionarySpec, read_diag_data_dictionary_spec_from_odx
 from .dataobjectproperty import DopBase
@@ -26,6 +29,7 @@ from .message import Message
 from .service import DiagService, read_diag_service_from_odx
 from .singleecujob import SingleEcuJob, read_single_ecu_job_from_odx
 from .structures import Request, Response, read_structure_from_odx
+from .specialdata import SpecialDataGroup, read_sdgs_from_odx
 
 # Defines priority of overriding objects
 PRIORITY_OF_DIAG_LAYER_TYPE = {
@@ -107,13 +111,13 @@ class DiagLayer:
                  diag_data_dictionary_spec: Optional[DiagDataDictionarySpec] = None,
                  communication_parameters:
                  Iterable[CommunicationParameterRef] = [],
-                 enable_candela_workarounds=True,
                  odxlinks=None,
                  additional_audiences=[],
                  functional_classes=[],
                  states=[],
                  state_transitions=[],
                  import_refs=[],
+                 sdgs=[],
                  ):
         logger.info(f"Initializing variant type {variant_type}")
         self.variant_type = variant_type
@@ -122,6 +126,7 @@ class DiagLayer:
         self.short_name = short_name
         self.long_name = long_name
         self.description = description
+        self.sdgs = sdgs
 
         # Requests and Responses
         self.requests = requests
@@ -164,9 +169,6 @@ class DiagLayer:
         if odxlinks is not None:
             self.finalize_init(odxlinks)
 
-        # specify whether enable work arounds for bugs of CANdela studio
-        self._enable_candela_workarounds = enable_candela_workarounds
-
     @property
     def services(self) -> NamedItemList[Union[DiagService, SingleEcuJob]]:
         """All services that this diagnostic layer offers including inherited services."""
@@ -204,7 +206,7 @@ class DiagLayer:
         """Construct a mapping from IDs to all objects that are contained in this diagnostic layer."""
         logger.info(f"Adding {self.odx_id} to odxlinks.")
 
-        odxlinks = {}
+        odxlinks = { self.odx_id: self }
 
         for obj in chain(self._local_services,
                          self._local_single_ecu_jobs,
@@ -217,12 +219,21 @@ class DiagLayer:
                          self.state_transitions):
             odxlinks[obj.odx_id] = obj
 
+        for obj in chain(self._local_services,
+                         self._local_single_ecu_jobs,
+                         self.requests,
+                         self.positive_responses,
+                         self.negative_responses,
+                         self.additional_audiences,
+                         self.functional_classes,
+                         self.sdgs,
+                         self.states,
+                         self.state_transitions):
+            odxlinks.update(obj._build_odxlinks())
+
         if self.local_diag_data_dictionary_spec:
-            odxlinks.update(
-                self.local_diag_data_dictionary_spec._build_odxlinks()
-            )
+            odxlinks.update(self.local_diag_data_dictionary_spec._build_odxlinks())
 
-        odxlinks[self.odx_id] = self
         return odxlinks
 
     def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
@@ -231,6 +242,9 @@ class DiagLayer:
         for pr in self.parent_refs:
             pr._resolve_references(odxlinks)
 
+        for sdg in self.sdgs:
+            sdg._resolve_references(odxlinks)
+
         services = sorted(self._compute_available_services_by_name(odxlinks).values(),
                           key=short_name_as_id)
         self._services = NamedItemList[Union[DiagService, SingleEcuJob]](
@@ -257,14 +271,15 @@ class DiagLayer:
             struct._resolve_references(self, odxlinks)
 
         local_diag_comms: Iterable[Union[DiagService, SingleEcuJob]] \
-            = (*self._local_services, *self._local_single_ecu_jobs)
-        for service in local_diag_comms:
-            service._resolve_references(odxlinks)
+            = [*self._local_services, *self._local_single_ecu_jobs]
+        for ldc in local_diag_comms:
+            ldc._resolve_references(odxlinks)
 
         if self.local_diag_data_dictionary_spec:
             self.local_diag_data_dictionary_spec._resolve_references(self,
                                                                      odxlinks)
 
+
     def __local_services_by_name(self, odxlinks: OdxLinkDatabase) -> Dict[str, Union[DiagService, SingleEcuJob]]:
         services_by_name: Dict[str, Union[DiagService, SingleEcuJob]] = {}
 
@@ -445,10 +460,14 @@ class DiagLayer:
                 f"None of the services {possible_services} could parse {response.hex()}.")
         return decoded_messages
 
-    def get_communication_parameter(self, name: str) \
+    def get_communication_parameter(self,
+                                    name: str,
+                                    protocol_name: Optional[str] = None) \
             -> Optional[CommunicationParameterRef]:
 
         cps = [cp for cp in self.communication_parameters if cp.short_name == name]
+        if protocol_name:
+            cps = [cp for cp in cps if cp.protocol_sn_ref in (None, protocol_name)]
         if len(cps) > 1:
             warnings.warn(f"Communication parameter `{name}` specified more "
                           f"than once. Using first occurence.", OdxWarning)
@@ -457,51 +476,136 @@ class DiagLayer:
 
         return cps[0]
 
-    def get_receive_id(self) -> Optional[int]:
+    def get_can_receive_id(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
         """CAN ID to which the ECU listens for diagnostic messages"""
-        com_param = self.get_communication_parameter("CP_UniqueRespIdTable")
+        com_param = self.get_communication_parameter("CP_UniqueRespIdTable",
+                                                     protocol_name)
         if com_param is None:
             return None
-        value = com_param.value
-        if not isinstance(value, dict):
-            return None
-            
-        return value.get('CP_CanPhysReqId', None)
 
-    def get_send_id(self) -> Optional[int]:
+        result = com_param.get_subvalue("CP_CanPhysReqId")
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return int(result)
+
+    @deprecated(reason="use get_can_receive_id()")
+    def get_receive_id(self) -> Optional[int]:
+        return self.get_can_receive_id()
+
+    def get_can_send_id(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
         """CAN ID to which the ECU sends replies to diagnostic messages"""
-        com_param = self.get_communication_parameter("CP_UniqueRespIdTable")
+        com_param = self.get_communication_parameter("CP_UniqueRespIdTable",
+                                                     protocol_name)
         if com_param is None:
             return None
-        value = com_param.value
-        if not isinstance(value, dict):
+
+        result = com_param.get_subvalue("CP_CanRespUSDTId")
+        if not result:
             return None
+        assert isinstance(result, str)
 
-        return value.get('CP_CanRespUSDTId', None)
+        return int(result)
 
-    def get_can_func_req_id(self) -> Optional[int]:
+    @deprecated(reason="use get_can_send_id()")
+    def get_send_id(self) -> Optional[int]:
+        return self.get_can_send_id()
+
+    def get_can_func_req_id(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
         """CAN Functional Request Id."""
-        com_param = self.get_communication_parameter("CP_CanFuncReqId")
-
+        com_param = self.get_communication_parameter("CP_CanFuncReqId",
+                                                     protocol_name)
         if com_param is None:
             return None
 
-        return int(com_param.value)
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
 
-    def get_logical_doip_address(self) -> Optional[int]:
-        """The logical DoIP address of the ECU."""
-        com_param = self.get_communication_parameter("CP_UniqueRespIdTable")
+        return int(result)
 
+    def get_doip_logical_gateway_address(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
+        """DoIp logical gateway address"""
+        com_param = self.get_communication_parameter("CP_DoIPLogicalGatewayAddress",
+                                                     protocol_name)
         if com_param is None:
             return None
 
-        value = com_param.value
-        if not isinstance(value, dict):
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return int(result)
+
+    def get_doip_logical_tester_address(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
+        """DoIp logical gateway address"""
+        com_param = self.get_communication_parameter("CP_DoIPLogicalTesterAddress",
+                                                     protocol_name)
+        if com_param is None:
             return None
 
-        return value.get('CP_CanPhysReqExtAddr', None)
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
 
-    def get_tester_present_time(self) -> Optional[float]:
+        return int(result)
+
+    def get_doip_logical_functional_address(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
+        """The logical functional DoIP address of the ECU."""
+        com_param = self.get_communication_parameter("CP_DoIPLogicalFunctionalAddress",
+                                                     protocol_name)
+        if com_param is None:
+            return None
+
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return int(result)
+
+    def get_doip_routing_activation_timeout(self, protocol_name: Optional[str] = None) \
+            -> Optional[float]:
+        """The timout for the DoIP routing activation request in seconds"""
+        com_param = self.get_communication_parameter("CP_DoIPRoutingActivationTimeout",
+                                                     protocol_name)
+        if com_param is None:
+            return None
+
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return float(result)/1e6
+
+    def get_doip_routing_activation_type(self, protocol_name: Optional[str] = None) \
+            -> Optional[int]:
+        """The  DoIP routing type"""
+        com_param = self.get_communication_parameter("CP_DoIPRoutingActivationType",
+                                                     protocol_name)
+        if com_param is None:
+            return None
+
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return int(result)
+
+    def get_tester_present_time(self, protocol_name: Optional[str] = None) \
+            -> Optional[float]:
         """Timeout on inactivity in seconds.
 
         This is defined by the communication parameter "CP_TesterPresentTime".
@@ -510,12 +614,17 @@ class DiagLayer:
         Description of the comparam: "Time between a response and the next subsequent tester present message
         (if no other request is sent to this ECU) in case of physically addressed requests."
         """
-        com_param = self.get_communication_parameter("CP_TesterPresentTime")
-
+        com_param = self.get_communication_parameter("CP_TesterPresentTime",
+                                                     protocol_name)
         if com_param is None:
-            return 3.0  # default specified by the standard
+            return None
 
-        return int(com_param.value) / 1e6
+        result = com_param.get_value()
+        if not result:
+            return None
+        assert isinstance(result, str)
+
+        return float(result)/1e6
 
     def __repr__(self) -> str:
         return f"""DiagLayer(variant_type={self.variant_type},
@@ -530,8 +639,7 @@ class DiagLayer:
           diag_comm_refs={self._diag_comm_refs},
           parent_refs={self.parent_refs},
           diag_data_dictionary_spec={self.local_diag_data_dictionary_spec},
-          communication_parameters={self._local_communication_parameters},
-          enable_candela_workarounds={self._enable_candela_workarounds})"""
+          communication_parameters={self._local_communication_parameters})"""
 
     def __str__(self) -> str:
         return f"DiagLayer('{self.short_name}', type='{self.variant_type}')"
@@ -557,14 +665,14 @@ def read_parent_ref_from_odx(et_element, doc_frags: List[OdxDocFragment]) \
     )
 
 
-def read_diag_layer_from_odx(et_element,
-                             doc_frags: List[OdxDocFragment],
-                             enable_candela_workarounds=True) \
+def read_diag_layer_from_odx(et_element: ElementTree.Element,
+                             doc_frags: List[OdxDocFragment]) \
         -> DiagLayer:
 
     variant_type = et_element.tag
 
     short_name = et_element.findtext("SHORT-NAME")
+    assert short_name is not None
     long_name = et_element.findtext("LONG-NAME")
     description = read_description_from_odx(et_element.find("DESC"))
 
@@ -643,33 +751,30 @@ def read_diag_layer_from_odx(et_element,
     import_refs = [OdxLinkRef.from_et(ref, doc_frags)
                    for ref in et_element.iterfind("IMPORT-REFS/IMPORT-REF")]
 
-    # TODO: Are UNIT-SPEC and SDGS needed?
+    sdgs = read_sdgs_from_odx(et_element.find("SDGS"), doc_frags)
 
     # Create DiagLayer
-    dl = DiagLayer(variant_type,
-                   odx_id,
-                   short_name,
-                   long_name=long_name,
-                   description=description,
-                   requests=requests,
-                   positive_responses=positive_responses,
-                   negative_responses=negative_responses,
-                   services=services,
-                   diag_comm_refs=diag_comm_refs,
-                   single_ecu_jobs=single_ecu_jobs,
-                   parent_refs=parent_refs,
-                   diag_data_dictionary_spec=diag_data_dictionary_spec,
-                   communication_parameters=com_params,
-                   additional_audiences=additional_audiences,
-                   functional_classes=functional_classes,
-                   states=states,
-                   state_transitions=state_transitions,
-                   enable_candela_workarounds=enable_candela_workarounds,
-                   import_refs=import_refs,
-                   )
-
-    return dl
-
+    return DiagLayer(variant_type,
+                     odx_id,
+                     short_name,
+                     long_name=long_name,
+                     description=description,
+                     requests=requests,
+                     positive_responses=positive_responses,
+                     negative_responses=negative_responses,
+                     services=services,
+                     diag_comm_refs=diag_comm_refs,
+                     single_ecu_jobs=single_ecu_jobs,
+                     parent_refs=parent_refs,
+                     diag_data_dictionary_spec=diag_data_dictionary_spec,
+                     communication_parameters=com_params,
+                     additional_audiences=additional_audiences,
+                     functional_classes=functional_classes,
+                     states=states,
+                     state_transitions=state_transitions,
+                     import_refs=import_refs,
+                     sdgs=sdgs,
+                     )
 
 class DiagLayerContainer:
     def __init__(self,
@@ -683,7 +788,8 @@ class DiagLayerContainer:
                  protocols: List[DiagLayer] = [],
                  functional_groups: List[DiagLayer] = [],
                  base_variants: List[DiagLayer] = [],
-                 ecu_variants: List[DiagLayer] = []
+                 ecu_variants: List[DiagLayer] = [],
+                 sdgs: List[SpecialDataGroup] = [],
                  ) -> None:
         self.odx_id = odx_id
         self.short_name = short_name
@@ -697,6 +803,7 @@ class DiagLayerContainer:
         self.functional_groups = functional_groups
         self.base_variants = base_variants
         self.ecu_variants = ecu_variants
+        self.sdgs = sdgs
 
         self._diag_layers = NamedItemList[DiagLayer](short_name_as_id, list(
             chain(self.ecu_shared_datas, self.protocols, self.functional_groups, self.base_variants, self.ecu_variants)))
@@ -712,6 +819,16 @@ class DiagLayerContainer:
             for cd in self.company_datas:
                 result.update(cd._build_odxlinks())
 
+        for dl in chain(self.ecu_shared_datas,
+                        self.protocols,
+                        self.functional_groups,
+                        self.base_variants,
+                        self.ecu_variants):
+            result.update(dl._build_odxlinks())
+
+        for sdg in self.sdgs:
+            result.update(sdg._build_odxlinks())
+
         return result
 
     def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
@@ -722,6 +839,16 @@ class DiagLayerContainer:
             for cd in self.company_datas:
                 cd._resolve_references(odxlinks)
 
+        for dl in chain(self.ecu_shared_datas,
+                        self.protocols,
+                        self.functional_groups,
+                        self.base_variants,
+                        self.ecu_variants):
+            dl._resolve_references(odxlinks)
+
+        for sdg in self.sdgs:
+            sdg._resolve_references(odxlinks)
+
     @property
     def diag_layers(self):
         return self._diag_layers
@@ -736,7 +863,7 @@ class DiagLayerContainer:
         return f"DiagLayerContainer('{self.short_name}')"
 
 
-def read_diag_layer_container_from_odx(et_element, enable_candela_workarounds=True):
+def read_diag_layer_container_from_odx(et_element):
     short_name = et_element.findtext("SHORT-NAME")
     assert short_name is not None
     long_name = et_element.findtext("LONG-NAME")
@@ -751,24 +878,19 @@ def read_diag_layer_container_from_odx(et_element, enable_candela_workarounds=Tr
     admin_data = read_admin_data_from_odx(et_element.find("ADMIN-DATA"), doc_frags)
     company_datas = read_company_datas_from_odx(et_element.find("COMPANY-DATAS"), doc_frags)
     ecu_shared_datas = [read_diag_layer_from_odx(dl_element,
-                                                 doc_frags,
-                                                 enable_candela_workarounds=enable_candela_workarounds)
+                                                 doc_frags)
                         for dl_element in et_element.iterfind("ECU-SHARED-DATAS/ECU-SHARED-DATA")]
     protocols = [read_diag_layer_from_odx(dl_element,
-                                          doc_frags,
-                                          enable_candela_workarounds=enable_candela_workarounds)
+                                          doc_frags)
                  for dl_element in et_element.iterfind("PROTOCOLS/PROTOCOL")]
     functional_groups = [read_diag_layer_from_odx(dl_element,
-                                                  doc_frags,
-                                                  enable_candela_workarounds=enable_candela_workarounds)
+                                                  doc_frags)
                          for dl_element in et_element.iterfind("FUNCTIONAL-GROUPS/FUNCTIONAL-GROUP")]
     base_variants = [read_diag_layer_from_odx(dl_element,
-                                              doc_frags,
-                                              enable_candela_workarounds=enable_candela_workarounds)
+                                              doc_frags)
                      for dl_element in et_element.iterfind("BASE-VARIANTS/BASE-VARIANT")]
     ecu_variants = [read_diag_layer_from_odx(dl_element,
-                                             doc_frags,
-                                             enable_candela_workarounds=enable_candela_workarounds)
+                                             doc_frags)
                     for dl_element in et_element.iterfind("ECU-VARIANTS/ECU-VARIANT")]
 
     return DiagLayerContainer(odx_id,
